﻿using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace Lfoedd
{
    internal class ColorTool : MVRScript
    {
        protected Atom _selectedAtom;
        protected UIDynamicColorPicker _clothingColorPicker;
        protected JSONStorableColor _selectedClothingColor;
        protected UIDynamicColorPicker _hairColorPicker;
        protected JSONStorableColor _selectedHairColor;
        protected UIDynamicButton _refreshButton;

        protected Dictionary<DAZClothingItemControl, JSONStorableBool> _clothingToggles = new Dictionary<DAZClothingItemControl, JSONStorableBool>();
        protected Dictionary<DAZClothingItemControl, UIDynamicToggle> _clothes = new Dictionary<DAZClothingItemControl, UIDynamicToggle>();
        protected List<DAZClothingItemControl> _selectedClothes = new List<DAZClothingItemControl>();

        protected Dictionary<DAZHairGroup, JSONStorableBool> _hairToggles = new Dictionary<DAZHairGroup, JSONStorableBool>();
        protected Dictionary<DAZHairGroup, UIDynamicToggle> _hairs = new Dictionary<DAZHairGroup, UIDynamicToggle>();
        protected List<DAZHairGroup> _selectedHairs = new List<DAZHairGroup>();

        public override void Init()
        {
            _selectedAtom = containingAtom;

            if (_selectedAtom.type != "Person")
            {
                SuperController.LogError("This script can only be attached to a Person atom.");
                return;
            }

            // Clothing
            _selectedClothingColor = new JSONStorableColor("Clothing color", HSVColorPicker.RGBToHSV(1f, 1f, 1f), UpdateColors);
            RegisterColor(_selectedClothingColor);
            _clothingColorPicker = CreateColorPicker(_selectedClothingColor, true);

            // Hair
            _selectedHairColor = new JSONStorableColor("Hair color", HSVColorPicker.RGBToHSV(1f, 1f, 1f), UpdateColors);
            RegisterColor(_selectedHairColor);
            _hairColorPicker = CreateColorPicker(_selectedHairColor, true);

            _refreshButton = CreateButton("Refresh", false);
            _refreshButton.height = 100;
            _refreshButton.button.onClick.AddListener(Refresh);

            Refresh();
        }

        private void Refresh()
        {
            foreach (var toggle in _clothes)
            {
                RemoveToggle(toggle.Value);
            }
            foreach (var toggle in _hairs)
            {
                RemoveToggle(toggle.Value);
            }

            // Clothes
            foreach (var clothingItem in GameObject.FindObjectsOfType<DAZClothingItemControl>().Where(x => x.containingAtom == _selectedAtom).ToList())
            {
                if (!_clothingToggles.ContainsKey(clothingItem))
                {
                    _clothingToggles.Add(clothingItem, new JSONStorableBool(clothingItem.name, false));
                    RegisterBool(_clothingToggles[clothingItem]);
                }

                var toggle = CreateToggle(_clothingToggles[clothingItem]);
                toggle.label = $"[Clothing] {clothingItem.clothingItem.displayName}";
                toggle.toggle.isOn = _selectedClothes.Contains(clothingItem);
                toggle.toggle.onValueChanged.AddListener(OnItemSelect);
                if (_clothes.ContainsKey(clothingItem)) _clothes[clothingItem] = toggle;
                else _clothes.Add(clothingItem, toggle);
            }

            // Hair
            foreach (var hairItem in GameObject.FindObjectsOfType<DAZHairGroup>().Where(x => x.containingAtom == _selectedAtom).ToList())
            {
                if (!_hairToggles.ContainsKey(hairItem))
                {
                    _hairToggles.Add(hairItem, new JSONStorableBool(hairItem.name, false));
                    RegisterBool(_hairToggles[hairItem]);
                }

                var toggle = CreateToggle(new JSONStorableBool(hairItem.displayName, false));
                toggle.label = $"[Hair] {hairItem.displayName}";
                toggle.toggle.isOn = _selectedHairs.Contains(hairItem);
                toggle.toggle.onValueChanged.AddListener(OnItemSelect);
                if (_hairs.ContainsKey(hairItem)) _hairs[hairItem] = toggle;
                else _hairs.Add(hairItem, toggle);
            }
        }

        // Params required for the callback, not actually used.
        private void UpdateColors(float h = 0f, float s = 0f, float v = 0f)
        {
            // Clothing
            foreach (var clothingItem in _selectedClothes)
            {
                foreach (var material in clothingItem.GetComponentsInChildren<MaterialOptions>())
                {
                    material.SetColor1(Color.HSVToRGB(_selectedClothingColor.val.H, _selectedClothingColor.val.S, _selectedClothingColor.val.V));
                }
            }

            // Hair
            foreach (var hairItem in _selectedHairs)
            {
                foreach (var hairControl in hairItem.GetComponentsInChildren<HairSimControl>())
                {
                    hairControl.SetColorParamValue("rootColor", _selectedHairColor.val);
                    hairControl.SetColorParamValue("tipColor", _selectedHairColor.val);
                    hairControl.SetColorParamValue("specularColor", _selectedHairColor.val);
                }
            }
        }

        private void OnItemSelect(bool toggled)
        {
            _selectedClothes.Clear();
            _selectedHairs.Clear();
            foreach (var toggle in _clothes)
            {
                if (toggle.Value.toggle.isOn) _selectedClothes.Add(toggle.Key);
            }
            foreach (var toggle in _hairs)
            {
                if (toggle.Value.toggle.isOn) _selectedHairs.Add(toggle.Key);
            }
            UpdateColors();
        }
    }
}